import { ExternalTokenizer, LRParser } from '@lezer/lr';
import { LRLanguage, LanguageSupport } from '@codemirror/language';
import { styleTags, tags } from '@lezer/highlight';

// This file was generated by lezer-generator. You probably shouldn't edit it.
const Comment = 1,
  MultilineComment = 2,
  Slash = 3,
  Backslash = 4,
  MultilineCommentToEOF = 5,
  Minus = 6,
  Number = 7;

/* Hand-written tokenizers for K tokens that can't be
   expressed by lezer's built-in tokenizer. */

const backslash = 92, e = 101, eof = -1, minus = 45, newline = 10, nine = 57, period = 46, slash = 47, space = 32, x = 120, zero = 48;

// This tokenizer is needed to determine whether a `/` is used as an adverb or is
// the beginning of a comment. This requires looking at the previous character and
// the position in the input. This tokenizer is also used to parse multiline comments.
const slashOrComment = new ExternalTokenizer(input => {
    if (input.next !== slash) {
        return;
    }

    const prev = input.peek(-1);
    const pos = input.pos;
    input.advance();

    if (prev !== space && prev !== newline && pos !== 0) {
        input.acceptToken(Slash);
        return;
    }

    if (input.next === newline && (prev === newline || pos === 0)) {
        // It's a multiline comment.
        while (input.next !== eof) {
            if (input.next === backslash && input.peek(-1) === newline) {
                input.advance();
                if (input.next === newline) {
                    break;
                }
            }
            input.advance();
        }
        input.acceptToken(MultilineComment);
        return;
    }

    while (input.next !== newline && input.next !== eof) {
        input.advance();
    }
    input.acceptToken(Comment);
});

// This tokenizer is needed to determine whether a `\` is used as an adverb or is
// the beginning of a comment to the end of the file. This requires looking at the previous
// character and the position in the input.
const backslashOrMultilineCommentToEOF = new ExternalTokenizer(input => {
    if (input.next !== backslash) {
        return;
    }

    const prev = input.peek(-1);
    const pos = input.pos;
    input.advance();

    if (input.next === backslash && (pos === 0 || prev === newline)) {
        input.advance();

        if (input.next === newline || input.next === eof) {
            input.advance();
            while (input.next !== eof) {
                input.advance();
            }
            input.acceptToken(MultilineCommentToEOF);
            return;
        }
        return;
    }

    input.acceptToken(Backslash);
});

// This tokenizer is needed to determine whether a `-` is used as an verb or
// the beginning of a number. This requires looking at the previous character and
// the position in the input. This tokenizer also includes full support for parsing
// floating point numbers and is a convenient way to avoid some overlapping tokens,
// such as `.e`, that would occur when using the built-in tokenizer.
const minusOrNumber = new ExternalTokenizer(input => {
    if (input.next >= zero && input.next <= nine && input.peek(1) !== x) {
        parseNumber(input);
        input.acceptToken(Number);
        return;
    }

    if (input.next !== minus) {
        return;
    }

    const prev = input.peek(-1);
    const pos = input.pos;

    input.advance();

    if (prev === space || prev === newline || pos === 0) {
        if (parseNumber(input)) {
            input.acceptToken(Number);
            return;
        }
    }

    input.acceptToken(Minus);
});

const parseNumber = input => {
    let isNumber;
    while (input.next >= zero && input.next <= nine) {
        isNumber = true;
        input.advance();
    }

    if (isNumber) {
        if (input.next === period) {
            input.advance();
        }

        while (input.next >= zero && input.next <= nine) {
            input.advance();
        }

        if (input.next === e) {
            input.advance();

            if (input.next === minus) {
                input.advance();
            }

            while (input.next >= zero && input.next <= nine) {
                input.advance();
            }
        }
    }
    return isNumber;
};

// This file was generated by lezer-generator. You probably shouldn't edit it.
const parser = LRParser.deserialize({
  version: 14,
  states: "$UQVQ_OOOOQ^'#C{'#C{OOQ^'#C|'#C|OOQ^'#C}'#C}OOQ^'#Cz'#CzOVQ_O'#CoOVQ_O'#CrO!TQ_O'#CyOOQ^'#Cy'#CyO#eQ_O'#CxOOQ^'#Cx'#CxOOQ^'#Ct'#CtQVQ_OOO$rQPO,59ZO$wQPO,59^OVQ_O'#ClOOQ^,59e,59eOVQ_O,59dOOQ^-E6r-E6rOOQ^1G.u1G.uOOQ^1G.x1G.xO$|QPO,59WOOQ^1G/O1G/OOOQ^1G.r1G.r",
  stateData: "%R~OkOS~OPYOQYORROSROTYOUQOVPOXPOYPOZPO[QO]RObTOeUO~O__OPmXQmXRmXSmXTmXUmXVmXXmXYmXZmX[mX]mXbmXemXgmXimXamXdmX^mX~OgaOPlXQlXRlXSlXTlXUlXVlXXlXYlXZlX[lX]lXblXelXilXalXdlX^lX~OacO~OddO~O^gO~O",
  goto: "#OrPPPPPPPPPPPPPPPPsPPvPPvP!OPPP!U!f!n!v!v!vR`V]WOTU[_aQ[ORb[SZO[Q]TQ^UQe_Rfa]XOTU[_a]VOTU[_a]SOTU[_a",
  nodeNames: "⚠ Comment MultilineComment Slash Backslash MultilineCommentToEOF Minus Number Program Name String Symbol Verb Adverb ] [ SquareBrackets ) ( Parentheses } { Function Delimiter",
  maxTerm: 33,
  nodeProps: [
    ["openedBy", 14,"[",17,"(",20,"{"],
    ["closedBy", 15,"]",18,")",21,"}"]
  ],
  skippedNodes: [0],
  repeatNodeCount: 1,
  tokenData: ")t~RuXY#fYZ#f]^#fpq#fqr#wrs$Ust#wtu#wuv#wvw#wwx%Pxy%^yz%cz{#w{|#w|}#w!O!P#w!Q!R%h!R![&V![!]#w!]!^&]!^!_#w!_!`#w!`!a#w!a!b#w!b!c#w!c!}&b!}#O&|#P#Q'R#Q#R#w#R#S#w#S#T'W#T#o&b#o#p)j#p#q#w#q#r)o#r#s#w~#kSk~XY#fYZ#f]^#fpq#f~#|P[~![!]$P~$UO[~~$XVOY$UYZ$nZr$Urs$ts#O$U#O#P$y#P~$U~$qPpq$U~$yOY~~$|PO~$U~%UP]~![!]%X~%^O]~~%cOb~~%hOa~~%kQ![!]#w#l#m%q~%vQY~!Q![%|#T#Z%|~&PQ!Q![%q#T#Z%q~&YP![!]#w~&bOg~~&gSX~!O!P&s!Q![&b!c!}&b#T#o&b~&vQ!c!}&b#T#o&b~'RO_~~'WO^~~']SZ~rs'i!Q!R(d!c!})O#T#o)O~'lVOY'iYZ(RZr'irs(Xs#O'i#O#P(^#P~'i~(UPpq'i~(^OZ~~(aPO~'i~(gP#l#m(j~(oQZ~!Q![(u#T#Z(u~(xQ!Q![(j#T#Z(j~)TSZ~!O!P)a!Q![)O!c!})O#T#o)O~)dQ!c!})O#T#o)O~)oOe~~)tOd~",
  tokenizers: [0, slashOrComment, backslashOrMultilineCommentToEOF, minusOrNumber],
  topRules: {"Program":[0,8]},
  tokenPrec: 0
});

const kLanguage = LRLanguage.define({
    parser: parser.configure({
        props: [
            styleTags({
                Number: tags.number,
                Comment: tags.lineComment,
                MultilineComment: tags.blockComment,
                MultilineCommentToEOF: tags.blockComment,
                String: tags.string,
                Name: tags.name,
                Symbol: tags.constant(tags.name),
                Delimiter: tags.separator,
                Adverb: tags.operator,
                Slash: tags.operator,
                Backslash: tags.operator,
                Minus: tags.operator,
                Verb: tags.operator,
                "( )": tags.paren,
                "[ ]": tags.squareBracket,
                "{ }": tags.bracket,
            })
        ]
    }),
    languageData: {
        commentTokens: { line: "/" },
        closeBrackets: { brackets: ["(", "[", "{", '"'] },
    }
});
function k() {
    return new LanguageSupport(kLanguage);
}

export { k, kLanguage };
